// Copyright (c) 2009-2015 The Bitcoin Core developers
// Copyright (c) 2015-2022 The Bitcoin Unlimited developers
// Distributed under the MIT software license, see the accompanying
// file COPYING or http://www.opensource.org/licenses/mit-license.php.

#ifndef NEXA_CLIENTVERSION_H
#define NEXA_CLIENTVERSION_H

#include "nexa-config.h"

/** Client flavor */

/**
 * Converts the parameter X to a string after macro replacement on X has been performed.
 * Don't merge these into one macro!
 */
#define STRINGIZE(X) DO_STRINGIZE(X)
#define DO_STRINGIZE(X) #X

//! Copyright string used in Windows .rc files
#define COPYRIGHT_STR "2009-" STRINGIZE(COPYRIGHT_YEAR) " " COPYRIGHT_HOLDERS_FINAL

/**
 * bitcoind-res.rc includes this file, but it cannot cope with real c++ code.
 * WINDRES_PREPROC is defined to indicate that its pre-processor is running.
 * Anything other than a define should be guarded below.
 */

#if !defined(WINDRES_PREPROC)

#include <string>
#include <vector>
extern const int CLIENT_VERSION; // BU make this a single constant instantiated in clientversion.cpp
extern const std::string CLIENT_NAME;
extern const std::string CLIENT_BUILD;
extern const std::string CLIENT_DATE;

extern std::string subverOverride; // Overrides the normal subver field with this string
extern bool fDisplayArchInSubver;

std::string FormatFullVersion();
std::string FormatSubVersion(const std::string &name, int nClientVersion, const std::vector<std::string> &comments);

#endif // WINDRES_PREPROC

#endif // NEXA_CLIENTVERSION_H
