// Copyright (c) 2009-2010 Satoshi Nakamoto
// Copyright (c) 2009-2014 The Bitcoin Core developers
// Copyright (c) 2015-2022 The Bitcoin Unlimited developers
// Distributed under the MIT software license, see the accompanying
// file COPYING or http://www.opensource.org/licenses/mit-license.php.

#ifndef NEXA_SCRIPT_SCRIPT_ERROR_H
#define NEXA_SCRIPT_SCRIPT_ERROR_H

typedef enum ScriptError_t
{
    SCRIPT_ERR_OK = 0,
    SCRIPT_ERR_UNKNOWN_ERROR,
    SCRIPT_ERR_EVAL_FALSE,
    SCRIPT_ERR_OP_RETURN,

    /* Max sizes */
    SCRIPT_ERR_SCRIPT_SIZE,
    SCRIPT_ERR_PUSH_SIZE,
    SCRIPT_ERR_OP_COUNT,
    SCRIPT_ERR_STACK_SIZE,
    SCRIPT_ERR_SIG_COUNT,
    SCRIPT_ERR_PUBKEY_COUNT,

    /* Operands checks */
    SCRIPT_ERR_INVALID_OPERAND_SIZE,
    SCRIPT_ERR_INVALID_NUMBER_RANGE,
    SCRIPT_ERR_IMPOSSIBLE_ENCODING,
    SCRIPT_ERR_INVALID_SPLIT_RANGE,
    SCRIPT_ERR_INVALID_BIT_COUNT,

    /* Failed verify operations */
    SCRIPT_ERR_VERIFY,
    SCRIPT_ERR_EQUALVERIFY,
    SCRIPT_ERR_CHECKMULTISIGVERIFY,
    SCRIPT_ERR_CHECKSIGVERIFY,
    SCRIPT_ERR_CHECKDATASIGVERIFY,
    SCRIPT_ERR_NUMEQUALVERIFY,

    /* Logical/Format/Canonical errors */
    SCRIPT_ERR_BAD_OPCODE,
    SCRIPT_ERR_DISABLED_OPCODE,
    SCRIPT_ERR_INVALID_STACK_OPERATION,
    SCRIPT_ERR_INVALID_ALTSTACK_OPERATION,
    SCRIPT_ERR_UNBALANCED_CONDITIONAL,

    /* Divisor errors */
    SCRIPT_ERR_DIV_BY_ZERO,
    SCRIPT_ERR_MOD_BY_ZERO,

    /* Bitfield errors */
    SCRIPT_ERR_INVALID_BITFIELD_SIZE,
    SCRIPT_ERR_INVALID_BIT_RANGE,

    /* CHECKLOCKTIMEVERIFY and CHECKSEQUENCEVERIFY */
    SCRIPT_ERR_NEGATIVE_LOCKTIME,
    SCRIPT_ERR_UNSATISFIED_LOCKTIME,

    /* BIP62 (Malleability) */
    SCRIPT_ERR_SIG_HASHTYPE,
    SCRIPT_ERR_SIG_DER,
    SCRIPT_ERR_MINIMALDATA,
    SCRIPT_ERR_SIG_PUSHONLY,
    SCRIPT_ERR_SIG_HIGH_S,
    SCRIPT_ERR_PUBKEYTYPE,
    SCRIPT_ERR_CLEANSTACK,
    SCRIPT_ERR_SIG_NULLFAIL,
    SCRIPT_ERR_MULTISIG_NULLFAIL,

    /* Schnorr */
    SCRIPT_ERR_SIG_BADLENGTH,
    SCRIPT_ERR_SIG_NONSCHNORR,
    SCRIPT_ERR_MUST_USE_FORKID,

    /* softfork safeness */
    SCRIPT_ERR_DISCOURAGE_UPGRADABLE_NOPS,

    /* misc */
    SCRIPT_ERR_NONCOMPRESSED_PUBKEY,
    SCRIPT_ERR_NUMBER_OVERFLOW,
    SCRIPT_ERR_NUMBER_BAD_ENCODING,
    SCRIPT_ERR_SIGCHECKS_LIMIT_EXCEEDED,

    /* TX introspection */
    SCRIPT_ERR_DATA_REQUIRED,
    SCRIPT_ERR_INVALID_TX_INPUT_INDEX,
    SCRIPT_ERR_INVALID_TX_OUTPUT_INDEX,

    /* nextchain */
    SCRIPT_ERR_TEMPLATE = 100,
    SCRIPT_ERR_EXEC_DEPTH_EXCEEDED,
    SCRIPT_ERR_EXEC_COUNT_EXCEEDED,
    SCRIPT_ERR_BAD_OPERATION_ON_TYPE,
    SCRIPT_ERR_STACK_LIMIT_EXCEEDED,
    SCRIPT_ERR_INVALID_STATE_SPECIFIER,
    SCRIPT_ERR_INITIAL_STATE,
    SCRIPT_ERR_INVALID_REGISTER,

    SCRIPT_ERR_STACK_BYTES, // Stack total length limit exceeded
    SCRIPT_ERR_PARSE, // Parsing the bytecode provided to op_parse failed
    SCRIPT_ERR_INVALID_JUMP,
    SCRIPT_ERR_INVALID_PARAMETER, // one of the opcode parameters is invalid

    SCRIPT_ERR_ERROR_COUNT
} ScriptError;


#define SCRIPT_ERR_LAST SCRIPT_ERR_ERROR_COUNT

const char *ScriptErrorString(const ScriptError error);

inline bool set_success(ScriptError *ret)
{
    if (ret)
        *ret = SCRIPT_ERR_OK;
    return true;
}

inline bool set_error(ScriptError *ret, const ScriptError serror)
{
    if (ret)
        *ret = serror;
    return false;
}

#endif // NEXA_SCRIPT_SCRIPT_ERROR_H
